#include "usecurses.h"
#include "connect.h"

static int timedout;
static void SubConnectOut(int signum)
{
	signum = timedout++;
}

int ConnectOut(const char *Server, int Port)
{
    struct hostent *HostName;
    struct sockaddr_in SocketName;
    int plug, p;
 
    if((plug = socket(AF_INET,SOCK_STREAM,0)) < 0)Error("socket");
 
    HostName = gethostbyname(Server);
    if(!HostName)Error("gethostbyname");
 
    SocketName.sin_family = AF_INET;
    SocketName.sin_port = htons(Port);
    memcpy(&SocketName.sin_addr, HostName->h_addr, HostName->h_length);
    signal(SIGABRT, SubConnectOut);
 
    p = fork();
    if(p < 0)Error("fork");
 
    timedout = 0;
    /* My lame timeout implementation that doesn't eat tons of CPU time
     * looping and doesn't clobber queued alarms */
    if (!p)
    {
        sleep(100);
        kill(getppid(),SIGABRT);
        exit(EXIT_SUCCESS);
    }
 
    for(;;)
    {
        if ((connect(plug,(struct sockaddr *)&SocketName,sizeof(SocketName))) < 0)
        {
            if(!((errno == EINTR)&&(!timedout)))
            {
                if (timedout)errno = ETIMEDOUT;
                kill(p, SIGKILL);
                signal(SIGABRT, SIG_DFL);
                waitpid(p,(int *)NULL,0);
                Error("connect");
            }
        } 
        else 
        {
            kill(p,SIGKILL);
            signal(SIGABRT,SIG_DFL);
            waitpid(p,(int *)NULL,0);
  
            return plug;
        }
    }
}

void SafeSend(int Socket, const void *Buf, int Bytes)
{
	int a=0;
	for(;;)
	{
		int b = send(Socket, &((char *)Buf)[a], Bytes, 0);
		if(b <= 0)return;
	    a += Bytes-b;
	    Bytes -= b;
	}
}

int WaitForConnection(int *Port, struct sockaddr_in *They, void(*Report)(void))
{
	int Plug, Sock, AddrLen;
	struct sockaddr_in MyAddr;
	
    if((Plug = socket(AF_INET,SOCK_STREAM, 0)) == -1)Error("socket");
ReBind:
    MyAddr.sin_port = htons(*Port);				/* network order */
	MyAddr.sin_family = AF_INET;				/* host byte order */
	MyAddr.sin_addr.s_addr = htonl(INADDR_ANY);	/* auto-fill with my IP, network long  */
    memset((void *)&(MyAddr.sin_zero), 0, 8);	/* zero the rest of the struct */
                       
    setsockopt(Plug, SOL_SOCKET, SO_REUSEADDR, &Sock, sizeof Sock);
    
	if(bind(Plug, (struct sockaddr *)&MyAddr, sizeof(struct sockaddr)))
	{
		if(errno==98)
		{
			(*Port)++;
			goto ReBind;
		}
		Error("bind");
	}
	
	if(Report)Report();
	
    if(listen(Plug, 10) == -1)Error("listen");
    
	AddrLen = sizeof(struct sockaddr_in);
	Sock = accept(Plug, (struct sockaddr *)They, (socklen_t *)&AddrLen);
	close(Plug);
	
	return Sock;
}
